@@ -1,7 +1,6 @@
 from __future__ import absolute_import, division, print_function
 
 import numpy as np
-from itertools import count
 import re
 
 try:
@@ -10,6 +9,7 @@ except ImportError:
     from toolz import concat, merge, unique
 
 from .core import Array, asarray, blockwise, getitem, apply_infer_dtype
+from .utils import meta_from_array
 from ..highlevelgraph import HighLevelGraph
 from ..core import flatten
 
@@ -18,13 +18,15 @@ from ..core import flatten
 # Modifications:
 #   - Allow for zero input arguments
 # See https://docs.scipy.org/doc/numpy/reference/c-api.generalized-ufuncs.html
-_DIMENSION_NAME = r'\w+'
-_CORE_DIMENSION_LIST = '(?:{0:}(?:,{0:})*,?)?'.format(_DIMENSION_NAME)
-_ARGUMENT = r'\({}\)'.format(_CORE_DIMENSION_LIST)
-_INPUT_ARGUMENTS = '(?:{0:}(?:,{0:})*,?)?'.format(_ARGUMENT)
-_OUTPUT_ARGUMENTS = '{0:}(?:,{0:})*'.format(_ARGUMENT)  # Use `'{0:}(?:,{0:})*,?'` if gufunc-
+_DIMENSION_NAME = r"\w+"
+_CORE_DIMENSION_LIST = "(?:{0:}(?:,{0:})*,?)?".format(_DIMENSION_NAME)
+_ARGUMENT = r"\({}\)".format(_CORE_DIMENSION_LIST)
+_INPUT_ARGUMENTS = "(?:{0:}(?:,{0:})*,?)?".format(_ARGUMENT)
+_OUTPUT_ARGUMENTS = "{0:}(?:,{0:})*".format(
+    _ARGUMENT
+)  # Use `'{0:}(?:,{0:})*,?'` if gufunc-
 # signature should be allowed for length 1 tuple returns
-_SIGNATURE = '^{0:}->{1:}$'.format(_INPUT_ARGUMENTS, _OUTPUT_ARGUMENTS)
+_SIGNATURE = "^{0:}->{1:}$".format(_INPUT_ARGUMENTS, _OUTPUT_ARGUMENTS)
 
 
 def _parse_gufunc_signature(signature):
@@ -43,15 +45,18 @@ def _parse_gufunc_signature(signature):
     of the form List[Tuple[str, ...]], except for one output. For one  output
     core dimension is not a list, but of the form Tuple[str, ...]
     """
-    signature = signature.replace(' ', '')
+    signature = signature.replace(" ", "")
     if not re.match(_SIGNATURE, signature):
-        raise ValueError('Not a valid gufunc signature: {}'.format(signature))
-    in_txt, out_txt = signature.split('->')
-    ins = [tuple(re.findall(_DIMENSION_NAME, arg))
-           for arg in re.findall(_ARGUMENT, in_txt)]
-    outs = [tuple(re.findall(_DIMENSION_NAME, arg))
-            for arg in re.findall(_ARGUMENT, out_txt)]
-    outs = outs[0] if ((len(outs) == 1) and (out_txt[-1] != ',')) else outs
+        raise ValueError("Not a valid gufunc signature: {}".format(signature))
+    in_txt, out_txt = signature.split("->")
+    ins = [
+        tuple(re.findall(_DIMENSION_NAME, arg)) for arg in re.findall(_ARGUMENT, in_txt)
+    ]
+    outs = [
+        tuple(re.findall(_DIMENSION_NAME, arg))
+        for arg in re.findall(_ARGUMENT, out_txt)
+    ]
+    outs = outs[0] if ((len(outs) == 1) and (out_txt[-1] != ",")) else outs
     return ins, outs
 
 
@@ -81,7 +86,9 @@ def _validate_normalize_axes(axes, axis, keepdims, input_coredimss, output_cored
     nout = 1 if not isinstance(output_coredimss, list) else len(output_coredimss)
 
     if axes is not None and axis is not None:
-        raise ValueError("Only one of `axis` or `axes` keyword arguments should be given")
+        raise ValueError(
+            "Only one of `axis` or `axes` keyword arguments should be given"
+        )
     if axes and not isinstance(axes, list):
         raise ValueError("`axes` has to be of type list")
 
@@ -101,10 +108,14 @@ def _validate_normalize_axes(axes, axis, keepdims, input_coredimss, output_cored
         if filtered_core_dims:
             cd0 = filtered_core_dims[0]
             if len(cd0) != 1:
-                raise ValueError("`axis` can be used only, if one core dimension is present")
+                raise ValueError(
+                    "`axis` can be used only, if one core dimension is present"
+                )
             for cd in filtered_core_dims:
                 if cd0 != cd:
-                    raise ValueError("To use `axis`, all core dimensions have to be equal")
+                    raise ValueError(
+                        "To use `axis`, all core dimensions have to be equal"
+                    )
 
     # Expand dafaults or axis
     if axes is None:
@@ -116,33 +127,50 @@ def _validate_normalize_axes(axes, axis, keepdims, input_coredimss, output_cored
         raise ValueError("`axes` argument has to be a list")
     axes = [(a,) if isinstance(a, int) else a for a in axes]
 
-    if (((nr_outputs_with_coredims == 0) and (nin != len(axes)) and (nin + nout != len(axes))) or
-            ((nr_outputs_with_coredims > 0) and (nin + nout != len(axes)))):
-        raise ValueError("The number of `axes` entries is not equal the number of input and output arguments")
+    if (
+        (nr_outputs_with_coredims == 0)
+        and (nin != len(axes))
+        and (nin + nout != len(axes))
+    ) or ((nr_outputs_with_coredims > 0) and (nin + nout != len(axes))):
+        raise ValueError(
+            "The number of `axes` entries is not equal the number of input and output arguments"
+        )
 
     # Treat outputs
     output_axes = axes[nin:]
-    output_axes = output_axes if output_axes else [tuple(range(-len(ocd), 0)) for ocd in output_coredimss]
+    output_axes = (
+        output_axes
+        if output_axes
+        else [tuple(range(-len(ocd), 0)) for ocd in output_coredimss]
+    )
     input_axes = axes[:nin]
 
     # Assert we have as many axes as output core dimensions
     for idx, (iax, icd) in enumerate(zip(input_axes, input_coredimss)):
         if len(iax) != len(icd):
-            raise ValueError("The number of `axes` entries for argument #{} is not equal "
-                             "the number of respective input core dimensions in signature"
-                             .format(idx))
+            raise ValueError(
+                "The number of `axes` entries for argument #{} is not equal "
+                "the number of respective input core dimensions in signature".format(
+                    idx
+                )
+            )
     if not keepdims:
         for idx, (oax, ocd) in enumerate(zip(output_axes, output_coredimss)):
             if len(oax) != len(ocd):
-                raise ValueError("The number of `axes` entries for argument #{} is not equal "
-                                 "the number of respective output core dimensions in signature"
-                                 .format(idx))
+                raise ValueError(
+                    "The number of `axes` entries for argument #{} is not equal "
+                    "the number of respective output core dimensions in signature".format(
+                        idx
+                    )
+                )
     else:
         if input_coredimss:
             icd0 = input_coredimss[0]
             for icd in input_coredimss:
                 if icd0 != icd:
-                    raise ValueError("To use `keepdims`, all core dimensions have to be equal")
+                    raise ValueError(
+                        "To use `keepdims`, all core dimensions have to be equal"
+                    )
             iax0 = input_axes[0]
             output_axes = [iax0 for _ in output_coredimss]
 
@@ -260,7 +288,7 @@ def apply_gufunc(func, signature, *args, **kwargs):
     # Input processing:
     ## Signature
     if not isinstance(signature, str):
-        raise TypeError('`signature` has to be of type string')
+        raise TypeError("`signature` has to be of type string")
     input_coredimss, output_coredimss = _parse_gufunc_signature(signature)
 
     ## Determine nout: nout = None for functions of one direct return; nout = int for return tuples
@@ -272,20 +300,28 @@ def apply_gufunc(func, signature, *args, **kwargs):
             tempfunc = np.vectorize(func, signature=signature)
         else:
             tempfunc = func
-        output_dtypes = apply_infer_dtype(tempfunc, args, kwargs, "apply_gufunc", "output_dtypes", nout)
+        output_dtypes = apply_infer_dtype(
+            tempfunc, args, kwargs, "apply_gufunc", "output_dtypes", nout
+        )
 
     if isinstance(output_dtypes, (tuple, list)):
         if nout is None:
             if len(output_dtypes) > 1:
-                raise ValueError(("Must specify single dtype or list of one dtype "
-                                  "for `output_dtypes` for function with one output"))
+                raise ValueError(
+                    (
+                        "Must specify single dtype or list of one dtype "
+                        "for `output_dtypes` for function with one output"
+                    )
+                )
             otypes = output_dtypes
             output_dtypes = output_dtypes[0]
         else:
             otypes = output_dtypes
     else:
         if nout is not None:
-            raise ValueError("Must specify tuple of dtypes for `output_dtypes` for function with multiple outputs")
+            raise ValueError(
+                "Must specify tuple of dtypes for `output_dtypes` for function with multiple outputs"
+            )
         otypes = [output_dtypes]
 
     ## Vectorize function, if required
@@ -297,15 +333,19 @@ def apply_gufunc(func, signature, *args, **kwargs):
         output_sizes = {}
 
     ## Axes
-    input_axes, output_axes = _validate_normalize_axes(axes, axis, keepdims, input_coredimss, output_coredimss)
+    input_axes, output_axes = _validate_normalize_axes(
+        axes, axis, keepdims, input_coredimss, output_coredimss
+    )
 
     # Main code:
     ## Cast all input arrays to dask
     args = [asarray(a) for a in args]
 
     if len(input_coredimss) != len(args):
-        ValueError("According to `signature`, `func` requires %d arguments, but %s given"
-                   % (len(input_coredimss), len(args)))
+        ValueError(
+            "According to `signature`, `func` requires %d arguments, but %s given"
+            % (len(input_coredimss), len(args))
+        )
 
     ## Axes: transpose input arguments
     transposed_args = []
@@ -323,11 +363,17 @@ def apply_gufunc(func, signature, *args, **kwargs):
     input_chunkss = [a.chunks for a in args]
     num_loopdims = [len(s) - len(cd) for s, cd in zip(input_shapes, input_coredimss)]
     max_loopdims = max(num_loopdims) if num_loopdims else None
-    core_input_shapes = [dict(zip(icd, s[n:])) for s, n, icd in zip(input_shapes, num_loopdims, input_coredimss)]
+    core_input_shapes = [
+        dict(zip(icd, s[n:]))
+        for s, n, icd in zip(input_shapes, num_loopdims, input_coredimss)
+    ]
     core_shapes = merge(*core_input_shapes)
     core_shapes.update(output_sizes)
 
-    loop_input_dimss = [tuple("__loopdim%d__" % d for d in range(max_loopdims - n, max_loopdims)) for n in num_loopdims]
+    loop_input_dimss = [
+        tuple("__loopdim%d__" % d for d in range(max_loopdims - n, max_loopdims))
+        for n in num_loopdims
+    ]
     input_dimss = [l + c for l, c in zip(loop_input_dimss, input_coredimss)]
 
     loop_output_dims = max(loop_input_dimss, key=len) if loop_input_dimss else tuple()
@@ -348,18 +394,28 @@ def apply_gufunc(func, signature, *args, **kwargs):
     for dim, sizes in dimsizess.items():
         #### Check that the arrays have same length for same dimensions or dimension `1`
         if set(sizes).union({1}) != {1, max(sizes)}:
-            raise ValueError("Dimension `'{}'` with different lengths in arrays".format(dim))
+            raise ValueError(
+                "Dimension `'{}'` with different lengths in arrays".format(dim)
+            )
         if not allow_rechunk:
             chunksizes = chunksizess[dim]
             #### Check if core dimensions consist of only one chunk
             if (dim in core_shapes) and (chunksizes[0][0] < core_shapes[dim]):
-                raise ValueError("Core dimension `'{}'` consists of multiple chunks. To fix, rechunk into a single \
+                raise ValueError(
+                    "Core dimension `'{}'` consists of multiple chunks. To fix, rechunk into a single \
 chunk along this dimension or set `allow_rechunk=True`, but beware that this may increase memory usage \
-significantly.".format(dim))
+significantly.".format(
+                        dim
+                    )
+                )
             #### Check if loop dimensions consist of same chunksizes, when they have sizes > 1
-            relevant_chunksizes = list(unique(c for s, c in zip(sizes, chunksizes) if s > 1))
+            relevant_chunksizes = list(
+                unique(c for s, c in zip(sizes, chunksizes) if s > 1)
+            )
             if len(relevant_chunksizes) > 1:
-                raise ValueError("Dimension `'{}'` with different chunksize present".format(dim))
+                raise ValueError(
+                    "Dimension `'{}'` with different chunksize present".format(dim)
+                )
 
     ## Apply function - use blockwise here
     arginds = list(concat(zip(args, input_dimss)))
@@ -367,20 +423,38 @@ significantly.".format(dim))
     ### Use existing `blockwise` but only with loopdims to enforce
     ### concatenation for coredims that appear also at the output
     ### Modifying `blockwise` could improve things here.
-    tmp = blockwise(
-        func,
-        loop_output_dims,
-        *arginds,
-        dtype=int,  # Only dummy dtype, anyone will do
-        concatenate=True,
-        **kwargs
-    )
+    try:
+        tmp = blockwise(  # First try to compute meta
+            func, loop_output_dims, *arginds, concatenate=True, **kwargs
+        )
+    except ValueError:
+        # If computing meta doesn't work, provide it explicitly based on
+        # provided dtypes
+        sample = arginds[0]._meta
+        if isinstance(output_dtypes, tuple):
+            meta = tuple(
+                meta_from_array(sample, dtype=odt)
+                for ocd, odt in zip(output_coredimss, output_dtypes)
+            )
+        else:
+            meta = tuple(
+                meta_from_array(sample, dtype=odt)
+                for ocd, odt in zip((output_coredimss,), (output_dtypes,))
+            )
+        tmp = blockwise(
+            func, loop_output_dims, *arginds, concatenate=True, meta=meta, **kwargs
+        )
+
+    if isinstance(tmp._meta, tuple):
+        metas = tmp._meta
+    else:
+        metas = (tmp._meta,)
 
     ## Prepare output shapes
     loop_output_shape = tmp.shape
     loop_output_chunks = tmp.chunks
     keys = list(flatten(tmp.__dask_keys__()))
-    name, token = keys[0][0].split('-')
+    name, token = keys[0][0].split("-")
 
     ### *) Treat direct output
     if nout is None:
@@ -389,19 +463,25 @@ significantly.".format(dim))
 
     ## Split output
     leaf_arrs = []
-    for i, ocd, odt, oax in zip(count(0), output_coredimss, output_dtypes, output_axes):
+    for i, (ocd, odt, oax, meta) in enumerate(
+        zip(output_coredimss, output_dtypes, output_axes, metas)
+    ):
         core_output_shape = tuple(core_shapes[d] for d in ocd)
         core_chunkinds = len(ocd) * (0,)
         output_shape = loop_output_shape + core_output_shape
         output_chunks = loop_output_chunks + core_output_shape
         leaf_name = "%s_%d-%s" % (name, i, token)
-        leaf_dsk = {(leaf_name,) + key[1:] + core_chunkinds: ((getitem, key, i) if nout else key) for key in keys}
+        leaf_dsk = {
+            (leaf_name,)
+            + key[1:]
+            + core_chunkinds: ((getitem, key, i) if nout else key)
+            for key in keys
+        }
         graph = HighLevelGraph.from_collections(leaf_name, leaf_dsk, dependencies=[tmp])
-        leaf_arr = Array(graph,
-                         leaf_name,
-                         chunks=output_chunks,
-                         shape=output_shape,
-                         dtype=odt)
+        meta = meta_from_array(meta, len(output_shape), dtype=odt)
+        leaf_arr = Array(
+            graph, leaf_name, chunks=output_chunks, shape=output_shape, meta=meta
+        )
 
         ### Axes:
         if keepdims:
@@ -496,7 +576,6 @@ class gufunc(object):
     >>> mean.compute().shape
     (10, 20)
 
-
     >>> a = da.random.normal(size=(   20,30), chunks=(10, 30))
     >>> b = da.random.normal(size=(10, 1,40), chunks=(5, 1, 40))
     >>> def outer_product(x, y):
@@ -511,6 +590,7 @@ class gufunc(object):
     .. [1] https://docs.scipy.org/doc/numpy/reference/ufuncs.html
     .. [2] https://docs.scipy.org/doc/numpy/reference/c-api.generalized-ufuncs.html
     """
+
     def __init__(self, pyfunc, **kwargs):
         self.pyfunc = pyfunc
         self.signature = kwargs.pop("signature", None)
@@ -540,20 +620,24 @@ class gufunc(object):
         Returns
         -------
         Single dask.array.Array or tuple of dask.array.Array
-        """.format(func=str(self.pyfunc), signature=self.signature)
+        """.format(
+            func=str(self.pyfunc), signature=self.signature
+        )
 
     def __call__(self, *args, **kwargs):
-        return apply_gufunc(self.pyfunc,
-                            self.signature,
-                            *args,
-                            vectorize=self.vectorize,
-                            axes=self.axes,
-                            axis=self.axis,
-                            keepdims=self.keepdims,
-                            output_sizes=self.output_sizes,
-                            output_dtypes=self.output_dtypes,
-                            allow_rechunk=self.allow_rechunk or kwargs.pop("allow_rechunk", False),
-                            **kwargs)
+        return apply_gufunc(
+            self.pyfunc,
+            self.signature,
+            *args,
+            vectorize=self.vectorize,
+            axes=self.axes,
+            axis=self.axis,
+            keepdims=self.keepdims,
+            output_sizes=self.output_sizes,
+            output_dtypes=self.output_dtypes,
+            allow_rechunk=self.allow_rechunk or kwargs.pop("allow_rechunk", False),
+            **kwargs
+        )
 
 
 def as_gufunc(signature=None, **kwargs):
@@ -636,12 +720,21 @@ def as_gufunc(signature=None, **kwargs):
     .. [1] https://docs.scipy.org/doc/numpy/reference/ufuncs.html
     .. [2] https://docs.scipy.org/doc/numpy/reference/c-api.generalized-ufuncs.html
     """
-    _allowedkeys = {"vectorize", "axes", "axis", "keepdims", "output_sizes", "output_dtypes", "allow_rechunk"}
+    _allowedkeys = {
+        "vectorize",
+        "axes",
+        "axis",
+        "keepdims",
+        "output_sizes",
+        "output_dtypes",
+        "allow_rechunk",
+    }
     if set(_allowedkeys).issubset(kwargs.keys()):
         raise TypeError("Unsupported keyword argument(s) provided")
 
     def _as_gufunc(pyfunc):
         return gufunc(pyfunc, signature=signature, **kwargs)
+
     _as_gufunc.__doc__ = """
         Decorator to make ``dask.array.gufunc``
         signature: ``'{signature}'``
@@ -654,5 +747,7 @@ def as_gufunc(signature=None, **kwargs):
         Returns
         -------
         ``dask.array.gufunc``
-        """.format(signature=signature)
+        """.format(
+        signature=signature
+    )
     return _as_gufunc
