@@ -5,9 +5,15 @@ from functools import wraps
 import numpy as np
 
 from ..base import normalize_token
-from .core import (concatenate_lookup, tensordot_lookup, map_blocks,
-                   asanyarray, blockwise)
+from .core import (
+    concatenate_lookup,
+    tensordot_lookup,
+    map_blocks,
+    asanyarray,
+    blockwise,
+)
 from .routines import _average
+from ..utils import derived_from
 
 
 @normalize_token.register(np.ma.masked_array)
@@ -21,10 +27,11 @@ def normalize_masked_array(x):
 @concatenate_lookup.register(np.ma.masked_array)
 def _concatenate(arrays, axis=0):
     out = np.ma.concatenate(arrays, axis=axis)
-    fill_values = [i.fill_value for i in arrays if hasattr(i, 'fill_value')]
+    fill_values = [i.fill_value for i in arrays if hasattr(i, "fill_value")]
     if any(isinstance(f, np.ndarray) for f in fill_values):
-        raise ValueError("Dask doesn't support masked array's with "
-                         "non-scalar `fill_value`s")
+        raise ValueError(
+            "Dask doesn't support masked array's with " "non-scalar `fill_value`s"
+        )
     if fill_values:
         # If all the fill_values are the same copy over the fill value
         fill_values = np.unique(fill_values)
@@ -101,14 +108,13 @@ def _tensordot(a, b, axes=2):
     return res.reshape(olda + oldb)
 
 
-@wraps(np.ma.filled)
+@derived_from(np.ma)
 def filled(a, fill_value=None):
     a = asanyarray(a)
     return a.map_blocks(np.ma.filled, fill_value=fill_value)
 
 
 def _wrap_masked(f):
-
     @wraps(f)
     def _(a, value):
         a = asanyarray(a)
@@ -128,110 +134,108 @@ masked_less_equal = _wrap_masked(np.ma.masked_less_equal)
 masked_not_equal = _wrap_masked(np.ma.masked_not_equal)
 
 
-@wraps(np.ma.masked_equal)
+@derived_from(np.ma)
 def masked_equal(a, value):
     a = asanyarray(a)
-    if getattr(value, 'shape', ()):
+    if getattr(value, "shape", ()):
         raise ValueError("da.ma.masked_equal doesn't support array `value`s")
     inds = tuple(range(a.ndim))
     return blockwise(np.ma.masked_equal, inds, a, inds, value, (), dtype=a.dtype)
 
 
-@wraps(np.ma.masked_invalid)
+@derived_from(np.ma)
 def masked_invalid(a):
     return asanyarray(a).map_blocks(np.ma.masked_invalid)
 
 
-@wraps(np.ma.masked_inside)
+@derived_from(np.ma)
 def masked_inside(x, v1, v2):
     x = asanyarray(x)
     return x.map_blocks(np.ma.masked_inside, v1, v2)
 
 
-@wraps(np.ma.masked_outside)
+@derived_from(np.ma)
 def masked_outside(x, v1, v2):
     x = asanyarray(x)
     return x.map_blocks(np.ma.masked_outside, v1, v2)
 
 
-@wraps(np.ma.masked_where)
+@derived_from(np.ma)
 def masked_where(condition, a):
-    cshape = getattr(condition, 'shape', ())
+    cshape = getattr(condition, "shape", ())
     if cshape and cshape != a.shape:
-        raise IndexError("Inconsistant shape between the condition and the "
-                         "input (got %s and %s)" % (cshape, a.shape))
+        raise IndexError(
+            "Inconsistant shape between the condition and the "
+            "input (got %s and %s)" % (cshape, a.shape)
+        )
     condition = asanyarray(condition)
     a = asanyarray(a)
     ainds = tuple(range(a.ndim))
     cinds = tuple(range(condition.ndim))
     return blockwise(
-        np.ma.masked_where,
-        ainds,
-        condition,
-        cinds,
-        a,
-        ainds,
-        dtype=a.dtype
+        np.ma.masked_where, ainds, condition, cinds, a, ainds, dtype=a.dtype
     )
 
 
-@wraps(np.ma.masked_values)
+@derived_from(np.ma)
 def masked_values(x, value, rtol=1e-05, atol=1e-08, shrink=True):
     x = asanyarray(x)
-    if getattr(value, 'shape', ()):
+    if getattr(value, "shape", ()):
         raise ValueError("da.ma.masked_values doesn't support array `value`s")
-    return map_blocks(np.ma.masked_values, x, value, rtol=rtol,
-                      atol=atol, shrink=shrink)
+    return map_blocks(
+        np.ma.masked_values, x, value, rtol=rtol, atol=atol, shrink=shrink
+    )
 
 
-@wraps(np.ma.fix_invalid)
+@derived_from(np.ma)
 def fix_invalid(a, fill_value=None):
     a = asanyarray(a)
     return a.map_blocks(np.ma.fix_invalid, fill_value=fill_value)
 
 
-@wraps(np.ma.getdata)
+@derived_from(np.ma)
 def getdata(a):
     a = asanyarray(a)
     return a.map_blocks(np.ma.getdata)
 
 
-@wraps(np.ma.getmaskarray)
+@derived_from(np.ma)
 def getmaskarray(a):
     a = asanyarray(a)
     return a.map_blocks(np.ma.getmaskarray)
 
 
 def _masked_array(data, mask=np.ma.nomask, **kwargs):
-    dtype = kwargs.pop('masked_dtype', None)
+    dtype = kwargs.pop("masked_dtype", None)
     return np.ma.masked_array(data, mask=mask, dtype=dtype, **kwargs)
 
 
-@wraps(np.ma.masked_array)
-def masked_array(data, mask=np.ma.nomask, fill_value=None,
-                 **kwargs):
+@derived_from(np.ma)
+def masked_array(data, mask=np.ma.nomask, fill_value=None, **kwargs):
     data = asanyarray(data)
     inds = tuple(range(data.ndim))
     arginds = [inds, data, inds]
 
-    if getattr(fill_value, 'shape', ()):
+    if getattr(fill_value, "shape", ()):
         raise ValueError("non-scalar fill_value not supported")
-    kwargs['fill_value'] = fill_value
+    kwargs["fill_value"] = fill_value
 
     if mask is not np.ma.nomask:
         mask = asanyarray(mask)
         if mask.size == 1:
             mask = mask.reshape((1,) * data.ndim)
         elif data.shape != mask.shape:
-            raise np.ma.MaskError("Mask and data not compatible: data shape "
-                                  "is %s, and mask shape is "
-                                  "%s." % (repr(data.shape), repr(mask.shape)))
+            raise np.ma.MaskError(
+                "Mask and data not compatible: data shape "
+                "is %s, and mask shape is "
+                "%s." % (repr(data.shape), repr(mask.shape))
+            )
         arginds.extend([mask, inds])
 
-    if 'dtype' in kwargs:
-        kwargs['masked_dtype'] = kwargs['dtype']
+    if "dtype" in kwargs:
+        kwargs["masked_dtype"] = kwargs["dtype"]
     else:
-        kwargs['dtype'] = data.dtype
+        kwargs["dtype"] = data.dtype
 
     return blockwise(_masked_array, *arginds, **kwargs)
 
@@ -243,10 +247,10 @@ def _set_fill_value(x, fill_value):
     return x
 
 
-@wraps(np.ma.set_fill_value)
+@derived_from(np.ma)
 def set_fill_value(a, fill_value):
     a = asanyarray(a)
-    if getattr(fill_value, 'shape', ()):
+    if getattr(fill_value, "shape", ()):
         raise ValueError("da.ma.set_fill_value doesn't support array `value`s")
     fill_value = np.ma.core._check_fill_value(fill_value, a.dtype)
     res = a.map_blocks(_set_fill_value, fill_value)
@@ -254,6 +258,6 @@ def set_fill_value(a, fill_value):
     a.name = res.name
 
 
-@wraps(np.ma.average)
+@derived_from(np.ma)
 def average(a, axis=None, weights=None, returned=False):
     return _average(a, axis, weights, returned, is_masked=True)
