@@ -307,23 +307,37 @@ def test_inner(shape1, shape2):
 
 
 @pytest.mark.parametrize(
-    "func1d_name, func1d",
+    "func1d_name, func1d, specify_output_props",
     [
-        ["ndim", lambda x: x.ndim],
-        ["sum", lambda x: x.sum()],
-        ["range", lambda x: [x.min(), x.max()]],
-        ["range2", lambda x: [[x.min(), x.max()], [x.max(), x.min()]]],
+        ["ndim", lambda x: x.ndim, False],
+        ["sum", lambda x: x.sum(), False],
+        ["range", lambda x: [x.min(), x.max()], False],
+        ["range2", lambda x: [[x.min(), x.max()], [x.max(), x.min()]], False],
+        ["cumsum", lambda x: np.cumsum(x), True],
     ],
 )
 @pytest.mark.parametrize(
-    "shape, axis",
+    "input_shape, axis",
     [[(10, 15, 20), 0], [(10, 15, 20), 1], [(10, 15, 20), 2], [(10, 15, 20), -1]],
 )
-def test_apply_along_axis(func1d_name, func1d, shape, axis):
-    a = np.random.randint(0, 10, shape)
-    d = da.from_array(a, chunks=(len(shape) * (5,)))
+def test_apply_along_axis(func1d_name, func1d, specify_output_props, input_shape, axis):
+    a = np.random.randint(0, 10, input_shape)
+    d = da.from_array(a, chunks=(len(input_shape) * (5,)))
+
+    output_shape = None
+    output_dtype = None
+
+    if specify_output_props:
+        slices = [0] * a.ndim
+        slices[axis] = slice(None)
+        slices = tuple(slices)
+        sample = np.array(func1d(a[slices]))
+        output_shape = sample.shape
+        output_dtype = sample.dtype
+
     assert_eq(
-        da.apply_along_axis(func1d, axis, d), np.apply_along_axis(func1d, axis, a)
+        da.apply_along_axis(func1d, axis, d, dtype=output_dtype, shape=output_shape),
+        np.apply_along_axis(func1d, axis, a),
     )
 
 
