@@ -3,7 +3,7 @@ import pytest
 import dask.array as da
 from ..utils import assert_eq
 
-xr = pytest.importorskip('xarray')
+xr = pytest.importorskip("xarray")
 
 
 def test_xarray():
