@@ -1,16 +1,22 @@
-from __future__ import absolute_import, division, print_function
-
 from operator import getitem
-from functools import partial, wraps
+from functools import partial
 
 import numpy as np
 from toolz import curry
 
 from .core import Array, elemwise, blockwise, apply_infer_dtype, asarray
+from .utils import empty_like_safe, IS_NEP18_ACTIVE
 from ..base import is_dask_collection, normalize_function
 from .. import core
 from ..highlevelgraph import HighLevelGraph
-from ..utils import skip_doctest, funcname
+from ..utils import (
+    skip_doctest,
+    funcname,
+    derived_from,
+    is_dataframe_like,
+    is_series_like,
+    is_index_like,
+)
 
 
 def __array_wrap__(numpy_ufunc, x, *args, **kwargs):
@@ -27,11 +33,15 @@ def wrap_elemwise(numpy_ufunc, array_wrap=False):
     """ Wrap up numpy function into dask.array """
 
     def wrapped(*args, **kwargs):
-        dsk = [arg for arg in args if hasattr(arg, '_elemwise')]
+        dsk = [arg for arg in args if hasattr(arg, "_elemwise")]
         if len(dsk) > 0:
-            if array_wrap:
-                return dsk[0]._elemwise(__array_wrap__, numpy_ufunc,
-                                        *args, **kwargs)
+            is_dataframe = (
+                is_dataframe_like(dsk[0])
+                or is_series_like(dsk[0])
+                or is_index_like(dsk[0])
+            )
+            if array_wrap and (is_dataframe or not IS_NEP18_ACTIVE):
+                return dsk[0]._elemwise(__array_wrap__, numpy_ufunc, *args, **kwargs)
             else:
                 return dsk[0]._elemwise(numpy_ufunc, *args, **kwargs)
         else:
@@ -45,16 +55,17 @@ def wrap_elemwise(numpy_ufunc, array_wrap=False):
 
 class da_frompyfunc(object):
     """A serializable `frompyfunc` object"""
+
     def __init__(self, func, nin, nout):
         self._ufunc = np.frompyfunc(func, nin, nout)
         self._func = func
         self.nin = nin
         self.nout = nout
         self._name = funcname(func)
-        self.__name__ = 'frompyfunc-%s' % self._name
+        self.__name__ = "frompyfunc-%s" % self._name
 
     def __repr__(self):
-        return 'da.frompyfunc<%s, %d, %d>' % (self._name, self.nin, self.nout)
+        return "da.frompyfunc<%s, %d, %d>" % (self._name, self.nin, self.nout)
 
     def __dask_tokenize__(self):
         return (normalize_function(self._func), self.nin, self.nout)
@@ -66,10 +77,11 @@ class da_frompyfunc(object):
         return self._ufunc(*args, **kwargs)
 
     def __getattr__(self, a):
-        if not a.startswith('_'):
+        if not a.startswith("_"):
             return getattr(self._ufunc, a)
-        raise AttributeError("%r object has no attribute "
-                             "%r" % (type(self).__name__, a))
+        raise AttributeError(
+            "%r object has no attribute " "%r" % (type(self).__name__, a)
+        )
 
     def __dir__(self):
         o = set(dir(type(self)))
@@ -78,7 +90,7 @@ class da_frompyfunc(object):
         return list(o)
 
 
-@wraps(np.frompyfunc)
+@derived_from(np)
 def frompyfunc(func, nin, nout):
     if nout > 1:
         raise NotImplementedError("frompyfunc with more than one output")
@@ -86,13 +98,22 @@ def frompyfunc(func, nin, nout):
 
 
 class ufunc(object):
-    _forward_attrs = {'nin', 'nargs', 'nout', 'ntypes', 'identity',
-                      'signature', 'types'}
+    _forward_attrs = {
+        "nin",
+        "nargs",
+        "nout",
+        "ntypes",
+        "identity",
+        "signature",
+        "types",
+    }
 
     def __init__(self, ufunc):
         if not isinstance(ufunc, (np.ufunc, da_frompyfunc)):
-            raise TypeError("must be an instance of `ufunc` or "
-                            "`da_frompyfunc`, got `%s" % type(ufunc).__name__)
+            raise TypeError(
+                "must be an instance of `ufunc` or "
+                "`da_frompyfunc`, got `%s" % type(ufunc).__name__
+            )
         self._ufunc = ufunc
         self.__name__ = ufunc.__name__
         copy_docstring(self, ufunc)
@@ -100,8 +121,9 @@ class ufunc(object):
     def __getattr__(self, key):
         if key in self._forward_attrs:
             return getattr(self._ufunc, key)
-        raise AttributeError("%r object has no attribute "
-                             "%r" % (type(self).__name__, key))
+        raise AttributeError(
+            "%r object has no attribute " "%r" % (type(self).__name__, key)
+        )
 
     def __dir__(self):
         return list(self._forward_attrs.union(dir(type(self)), self.__dict__))
@@ -110,14 +132,15 @@ class ufunc(object):
         return repr(self._ufunc)
 
     def __call__(self, *args, **kwargs):
-        dsks = [arg for arg in args if hasattr(arg, '_elemwise')]
+        dsks = [arg for arg in args if hasattr(arg, "_elemwise")]
         if len(dsks) > 0:
             for dsk in dsks:
                 result = dsk._elemwise(self._ufunc, *args, **kwargs)
                 if type(result) != type(NotImplemented):
                     return result
-            raise TypeError("Parameters of such types "
-                            "are not supported by " + self.__name__)
+            raise TypeError(
+                "Parameters of such types " "are not supported by " + self.__name__
+            )
         else:
             return self._ufunc(*args, **kwargs)
 
@@ -125,40 +148,49 @@ class ufunc(object):
     def outer(self, A, B, **kwargs):
         if self.nin != 2:
             raise ValueError("outer product only supported for binary functions")
-        if 'out' in kwargs:
+        if "out" in kwargs:
             raise ValueError("`out` kwarg not supported")
 
         A_is_dask = is_dask_collection(A)
         B_is_dask = is_dask_collection(B)
         if not A_is_dask and not B_is_dask:
             return self._ufunc.outer(A, B, **kwargs)
-        elif (A_is_dask and not isinstance(A, Array) or
-              B_is_dask and not isinstance(B, Array)):
-            raise NotImplementedError("Dask objects besides `dask.array.Array` "
-                                      "are not supported at this time.")
+        elif (
+            A_is_dask
+            and not isinstance(A, Array)
+            or B_is_dask
+            and not isinstance(B, Array)
+        ):
+            raise NotImplementedError(
+                "Dask objects besides `dask.array.Array` "
+                "are not supported at this time."
+            )
 
         A = asarray(A)
         B = asarray(B)
         ndim = A.ndim + B.ndim
         out_inds = tuple(range(ndim))
-        A_inds = out_inds[:A.ndim]
-        B_inds = out_inds[A.ndim:]
+        A_inds = out_inds[: A.ndim]
+        B_inds = out_inds[A.ndim :]
 
-        dtype = apply_infer_dtype(self._ufunc.outer, [A, B], kwargs,
-                                  'ufunc.outer', suggest_dtype=False)
+        dtype = apply_infer_dtype(
+            self._ufunc.outer, [A, B], kwargs, "ufunc.outer", suggest_dtype=False
+        )
 
-        if 'dtype' in kwargs:
-            func = partial(self._ufunc.outer, dtype=kwargs.pop('dtype'))
+        if "dtype" in kwargs:
+            func = partial(self._ufunc.outer, dtype=kwargs.pop("dtype"))
         else:
             func = self._ufunc.outer
 
         return blockwise(
             func,
             out_inds,
-            A, A_inds,
-            B, B_inds,
+            A,
+            A_inds,
+            B,
+            B_inds,
             dtype=dtype,
-            token=self.__name__ + '.outer',
+            token=self.__name__ + ".outer",
             **kwargs
         )
 
@@ -219,6 +251,8 @@ less = ufunc(np.less)
 less_equal = ufunc(np.less_equal)
 not_equal = ufunc(np.not_equal)
 equal = ufunc(np.equal)
+isneginf = partial(equal, -np.inf)
+isposinf = partial(equal, np.inf)
 logical_and = ufunc(np.logical_and)
 logical_or = ufunc(np.logical_or)
 logical_xor = ufunc(np.logical_xor)
@@ -264,8 +298,6 @@ absolute = ufunc(np.absolute)
 clip = wrap_elemwise(np.clip)
 isreal = wrap_elemwise(np.isreal, array_wrap=True)
 iscomplex = wrap_elemwise(np.iscomplex, array_wrap=True)
-isneginf = wrap_elemwise(np.isneginf, array_wrap=True)
-isposinf = wrap_elemwise(np.isposinf, array_wrap=True)
 real = wrap_elemwise(np.real, array_wrap=True)
 imag = wrap_elemwise(np.imag, array_wrap=True)
 fix = wrap_elemwise(np.fix, array_wrap=True)
@@ -277,7 +309,7 @@ nan_to_num = wrap_elemwise(np.nan_to_num, array_wrap=True)
 @copy_docstring(source=np.angle)
 def angle(x, deg=0):
     deg = bool(deg)
-    if hasattr(x, '_elemwise'):
+    if hasattr(x, "_elemwise"):
         return x._elemwise(__array_wrap__, np.angle, x, deg)
     return np.angle(x, deg=deg)
 
@@ -286,22 +318,24 @@ def angle(x, deg=0):
 def frexp(x):
     # Not actually object dtype, just need to specify something
     tmp = elemwise(np.frexp, x, dtype=object)
-    left = 'mantissa-' + tmp.name
-    right = 'exponent-' + tmp.name
-    ldsk = {(left,) + key[1:]: (getitem, key, 0)
-            for key in core.flatten(tmp.__dask_keys__())}
-    rdsk = {(right,) + key[1:]: (getitem, key, 1)
-            for key in core.flatten(tmp.__dask_keys__())}
+    left = "mantissa-" + tmp.name
+    right = "exponent-" + tmp.name
+    ldsk = {
+        (left,) + key[1:]: (getitem, key, 0)
+        for key in core.flatten(tmp.__dask_keys__())
+    }
+    rdsk = {
+        (right,) + key[1:]: (getitem, key, 1)
+        for key in core.flatten(tmp.__dask_keys__())
+    }
 
-    a = np.empty((1, ), dtype=x.dtype)
+    a = empty_like_safe(getattr(x, "_meta", x), shape=(1,) * x.ndim, dtype=x.dtype)
     l, r = np.frexp(a)
-    ldt = l.dtype
-    rdt = r.dtype
 
     graph = HighLevelGraph.from_collections(left, ldsk, dependencies=[tmp])
-    L = Array(graph, left, chunks=tmp.chunks, dtype=ldt)
+    L = Array(graph, left, chunks=tmp.chunks, meta=l)
     graph = HighLevelGraph.from_collections(right, rdsk, dependencies=[tmp])
-    R = Array(graph, right, chunks=tmp.chunks, dtype=rdt)
+    R = Array(graph, right, chunks=tmp.chunks, meta=r)
     return L, R
 
 
@@ -309,20 +343,29 @@ def frexp(x):
 def modf(x):
     # Not actually object dtype, just need to specify something
     tmp = elemwise(np.modf, x, dtype=object)
-    left = 'modf1-' + tmp.name
-    right = 'modf2-' + tmp.name
-    ldsk = {(left,) + key[1:]: (getitem, key, 0)
-            for key in core.flatten(tmp.__dask_keys__())}
-    rdsk = {(right,) + key[1:]: (getitem, key, 1)
-            for key in core.flatten(tmp.__dask_keys__())}
+    left = "modf1-" + tmp.name
+    right = "modf2-" + tmp.name
+    ldsk = {
+        (left,) + key[1:]: (getitem, key, 0)
+        for key in core.flatten(tmp.__dask_keys__())
+    }
+    rdsk = {
+        (right,) + key[1:]: (getitem, key, 1)
+        for key in core.flatten(tmp.__dask_keys__())
+    }
 
-    a = np.empty((1,), dtype=x.dtype)
+    a = empty_like_safe(getattr(x, "_meta", x), shape=(1,) * x.ndim, dtype=x.dtype)
     l, r = np.modf(a)
-    ldt = l.dtype
-    rdt = r.dtype
 
     graph = HighLevelGraph.from_collections(left, ldsk, dependencies=[tmp])
-    L = Array(graph, left, chunks=tmp.chunks, dtype=ldt)
+    L = Array(graph, left, chunks=tmp.chunks, meta=l)
     graph = HighLevelGraph.from_collections(right, rdsk, dependencies=[tmp])
-    R = Array(graph, right, chunks=tmp.chunks, dtype=rdt)
+    R = Array(graph, right, chunks=tmp.chunks, meta=r)
     return L, R
+
+
+@copy_docstring(source=np.divmod)
+def divmod(x, y):
+    res1 = x // y
+    res2 = x % y
+    return res1, res2
