@@ -2,6 +2,7 @@
 from __future__ import absolute_import, division, print_function
 
 from itertools import repeat
+import multiprocessing
 import math
 import os
 import random
@@ -643,15 +644,18 @@ def test_read_text_encoding():
 
 def test_read_text_large_gzip():
     with tmpfile("gz") as fn:
+        data = b"Hello, world!\n" * 100
         f = GzipFile(fn, "wb")
-        f.write(b"Hello, world!\n" * 100)
+        f.write(data)
         f.close()
 
         with pytest.raises(ValueError):
+            # not allowed blocks when compressed
             db.read_text(fn, blocksize=50, linedelimiter="\n")
 
-        c = db.read_text(fn)
+        c = db.read_text(fn, blocksize=None)
         assert c.npartitions == 1
+        assert "".join(c.compute()) == data.decode()
 
 
 @pytest.mark.slow
@@ -1366,10 +1370,15 @@ def test_repeated_groupby():
 def test_temporary_directory(tmpdir):
     b = db.range(10, npartitions=4)
 
-    with dask.config.set(temporary_directory=str(tmpdir)):
-        b2 = b.groupby(lambda x: x % 2)
-        b2.compute()
-        assert any(fn.endswith(".partd") for fn in os.listdir(str(tmpdir)))
+    # We use a pool to avoid a race condition between the pool close
+    # cleaning up files, and the assert below.
+    pool = multiprocessing.Pool(4)
+
+    with pool:
+        with dask.config.set(temporary_directory=str(tmpdir), pool=pool):
+            b2 = b.groupby(lambda x: x % 2)
+            b2.compute()
+            assert any(fn.endswith(".partd") for fn in os.listdir(str(tmpdir)))
 
 
 def test_empty_bag():
@@ -1382,7 +1391,10 @@ def test_empty_bag():
 
 def test_bag_paths():
     b = db.from_sequence(["abc", "123", "xyz"], npartitions=2)
-    assert b.to_textfiles("foo*") == ["foo0", "foo1"]
+    paths = b.to_textfiles("foo*")
+    assert paths[0].endswith("foo0")
+    assert paths[1].endswith("foo1")
+
     os.remove("foo0")
     os.remove("foo1")
 
