@@ -2,9 +2,7 @@ from __future__ import absolute_import, division, print_function
 
 from contextlib import contextmanager
 
-from .context import _globals
-
-__all__ = ['Callback', 'add_callbacks']
+__all__ = ["Callback", "add_callbacks"]
 
 
 class Callback(object):
@@ -47,7 +45,11 @@ class Callback(object):
     ...     x.compute()  # doctest: +SKIP
     """
 
-    def __init__(self, start=None, start_state=None, pretask=None, posttask=None, finish=None):
+    active = set()
+
+    def __init__(
+        self, start=None, start_state=None, pretask=None, posttask=None, finish=None
+    ):
         if start:
             self._start = start
         if start_state:
@@ -61,7 +63,7 @@ class Callback(object):
 
     @property
     def _callback(self):
-        fields = ['_start', '_start_state', '_pretask', '_posttask', '_finish']
+        fields = ["_start", "_start_state", "_pretask", "_posttask", "_finish"]
         return tuple(getattr(self, i, None) for i in fields)
 
     def __enter__(self):
@@ -73,10 +75,10 @@ class Callback(object):
         self._cm.__exit__(*args)
 
     def register(self):
-        _globals['callbacks'].add(self._callback)
+        Callback.active.add(self._callback)
 
     def unregister(self):
-        _globals['callbacks'].remove(self._callback)
+        Callback.active.remove(self._callback)
 
 
 def unpack_callbacks(cbs):
@@ -95,12 +97,12 @@ def local_callbacks(callbacks=None):
     This means that only the outermost scheduler will use global callbacks."""
     global_callbacks = callbacks is None
     if global_callbacks:
-        callbacks, _globals['callbacks'] = _globals['callbacks'], set()
+        callbacks, Callback.active = Callback.active, set()
     try:
         yield callbacks or ()
     finally:
         if global_callbacks:
-            _globals['callbacks'] = callbacks
+            Callback.active = callbacks
 
 
 def normalize_callback(cb):
@@ -128,13 +130,14 @@ class add_callbacks(object):
     >>> with add_callbacks(callbacks):    # doctest: +SKIP
     ...     res.compute()
     """
+
     def __init__(self, *callbacks):
         self.callbacks = [normalize_callback(c) for c in callbacks]
-        _globals['callbacks'].update(self.callbacks)
+        Callback.active.update(self.callbacks)
 
     def __enter__(self):
         return
 
     def __exit__(self, type, value, traceback):
         for c in self.callbacks:
-            _globals['callbacks'].discard(c)
+            Callback.active.discard(c)
