@@ -36,6 +36,7 @@ from ..utils import is_index_like as dask_is_index_like
 PANDAS_VERSION = LooseVersion(pd.__version__)
 PANDAS_GT_0230 = PANDAS_VERSION >= LooseVersion("0.23.0")
 PANDAS_GT_0240 = PANDAS_VERSION >= LooseVersion("0.24.0rc1")
+PANDAS_GT_0250 = PANDAS_VERSION >= LooseVersion("0.25.0")
 HAS_INT_NA = PANDAS_GT_0240
 
 
@@ -201,14 +202,14 @@ def has_known_categories(x):
     x : Series or CategoricalIndex
     """
     x = getattr(x, "_meta", x)
-    if isinstance(x, pd.Series):
+    if is_series_like(x):
         return UNKNOWN_CATEGORIES not in x.cat.categories
-    elif isinstance(x, pd.CategoricalIndex):
+    elif is_index_like(x) and hasattr(x, "categories"):
         return UNKNOWN_CATEGORIES not in x.categories
     raise TypeError("Expected Series or CategoricalIndex")
 
 
-def strip_unknown_categories(x):
+def strip_unknown_categories(x, just_drop_unknown=False):
     """Replace any unknown categoricals with empty categoricals.
 
     Useful for preventing ``UNKNOWN_CATEGORIES`` from leaking into results.
@@ -221,7 +222,10 @@ def strip_unknown_categories(x):
                 cats = cat_mask[cat_mask].index
                 for c in cats:
                     if not has_known_categories(x[c]):
-                        x[c].cat.set_categories([], inplace=True)
+                        if just_drop_unknown:
+                            x[c].cat.remove_categories(UNKNOWN_CATEGORIES, inplace=True)
+                        else:
+                            x[c].cat.set_categories([], inplace=True)
         elif isinstance(x, pd.Series):
             if is_categorical_dtype(x.dtype) and not has_known_categories(x):
                 x.cat.set_categories([], inplace=True)
