@@ -1018,7 +1018,3 @@ def test_num_workers_config(scheduler):
     workers = {i.worker_id for i in prof.results}
 
     assert len(workers) == num_workers
-
-
-if sys.version_info >= (3, 5):
-    from dask.tests.py3_test_await import *  # noqa F401
