@@ -1075,8 +1075,8 @@ def is_dataframe_like(df):
     typ = type(df)
     return (
         all(hasattr(typ, name) for name in ("groupby", "head", "merge", "mean"))
-        and all(hasattr(df, name) for name in ("dtypes",))
-        and not any(hasattr(typ, name) for name in ("value_counts", "dtype"))
+        and all(hasattr(df, name) for name in ("dtypes", "columns"))
+        and not any(hasattr(typ, name) for name in ("name", "dtype"))
     )
 
 
